.grid-bg-light.svelte-rx3fj1{background-size:30px 30px;background-image:radial-gradient(circle,#e4e4e48c 2px,#0000 2px)}.grid-bg-dark.svelte-rx3fj1{background-size:30px 30px;background-image:radial-gradient(circle,#46464646 2px,#0000 2px)}
