import"../chunks/DsnmJJEf.js";import{i as x}from"../chunks/Cr2l1Gk0.js";import{l as _,o as h,C as $,D as k,t as u,n as q,q as A,F as o,G as n,I as C,J as D,K as E,L as m}from"../chunks/BLXpcXkd.js";import{i as F}from"../chunks/8VNsxgWr.js";import{s as G,a as H}from"../chunks/BreKPyJ7.js";import{s as I,g as J}from"../chunks/BmybysiL.js";const K=()=>{const s=I;return{page:{subscribe:s.page.subscribe},navigating:{subscribe:s.navigating.subscribe},updated:s.updated}},L={subscribe(s){return K().page.subscribe(s)}};var M=A('<div class="container mx-auto p-8"><h1 class="mb-4 text-2xl font-bold"> </h1> <p class="mb-4"> </p> <a href="/" class="text-blue-500 hover:underline">Return to Home</a></div>');function B(s,i){_(i,!1);const[b,f]=G(),t=()=>H(L,"$page",b);h(()=>{t().status===404&&J("/")}),x();var p=$(),g=k(p);{var l=e=>{var r=M(),a=o(r),d=o(a);n(a);var c=C(a,2),v=o(c,!0);n(c),D(2),n(r),E(()=>{m(d,`Error ${t().status??""}`),m(v,t().error?.message||"An unexpected error occurred")}),u(e,r)};F(g,e=>{t().status!==404&&e(l)})}u(s,p),q(),f()}export{B as component};
