import"../chunks/DsnmJJEf.js";import{i as e}from"../chunks/Cr2l1Gk0.js";import{l as n,o as i,n as s}from"../chunks/BLXpcXkd.js";import{g as p}from"../chunks/BmybysiL.js";const c=o=>{const t=o.hash.split("/");let a="edit",r="";return t.length>2&&(a=t[1],r=`#${t[2]}`),`/${a}${o.search}${r}`};function u(o,t){n(t,!1),i(async()=>{await p(c(window.location),{replaceState:!0})}),e(),s()}export{u as component};
