import{W as B,X as N,P as A,Y as j,Z as x,_ as E,$ as K,a0 as D,z as b,a1 as U,x as Y,a2 as R,a3 as q,T as C,a4 as M,a5 as z,a6 as W,a7 as Z,a8 as G,a9 as P,aa as T,ab as X,ac as F,ad as H,ae as y,af as I}from"./BLXpcXkd.js";import{c as J}from"./BreKPyJ7.js";function L(e,r){return e===r||e?.[x]===r}function ie(e={},r,n,t){return B(()=>{var s,i;return N(()=>{s=i,i=[],A(()=>{e!==n(...i)&&(r(e,...i),s&&L(n(...s),e)&&r(null,...s))})}),()=>{j(()=>{i&&L(n(...i),e)&&r(null,...i)})}}),e}const Q={get(e,r){if(!e.exclude.includes(r))return e.props[r]},set(e,r){return!1},getOwnPropertyDescriptor(e,r){if(!e.exclude.includes(r)&&r in e.props)return{enumerable:!0,configurable:!0,value:e.props[r]}},has(e,r){return e.exclude.includes(r)?!1:r in e.props},ownKeys(e){return Reflect.ownKeys(e.props).filter(r=>!e.exclude.includes(r))}};function ue(e,r,n){return new Proxy({props:e,exclude:r},Q)}const V={get(e,r){if(!e.exclude.includes(r))return b(e.version),r in e.special?e.special[r]():e.props[r]},set(e,r,n){if(!(r in e.special)){var t=R;try{I(e.parent_effect),e.special[r]=ee({get[r](){return e.props[r]}},r,D)}finally{I(t)}}return e.special[r](n),y(e.version),!0},getOwnPropertyDescriptor(e,r){if(!e.exclude.includes(r)&&r in e.props)return{enumerable:!0,configurable:!0,value:e.props[r]}},deleteProperty(e,r){return e.exclude.includes(r)||(e.exclude.push(r),y(e.version)),!0},has(e,r){return e.exclude.includes(r)?!1:r in e.props},ownKeys(e){return Reflect.ownKeys(e.props).filter(r=>!e.exclude.includes(r))}};function ae(e,r){return new Proxy({props:e,exclude:r,special:{},version:H(0),parent_effect:R},V)}const k={get(e,r){let n=e.props.length;for(;n--;){let t=e.props[n];if(P(t)&&(t=t()),typeof t=="object"&&t!==null&&r in t)return t[r]}},set(e,r,n){let t=e.props.length;for(;t--;){let s=e.props[t];P(s)&&(s=s());const i=E(s,r);if(i&&i.set)return i.set(n),!0}return!1},getOwnPropertyDescriptor(e,r){let n=e.props.length;for(;n--;){let t=e.props[n];if(P(t)&&(t=t()),typeof t=="object"&&t!==null&&r in t){const s=E(t,r);return s&&!s.configurable&&(s.configurable=!0),s}}},has(e,r){if(r===x||r===T)return!1;for(let n of e.props)if(P(n)&&(n=n()),n!=null&&r in n)return!0;return!1},ownKeys(e){const r=[];for(let n of e.props)if(P(n)&&(n=n()),!!n){for(const t in n)r.includes(t)||r.push(t);for(const t of Object.getOwnPropertySymbols(n))r.includes(t)||r.push(t)}return r}};function le(...e){return new Proxy({props:e},k)}function ee(e,r,n,t){var s=!W||(n&Z)!==0,i=(n&z)!==0,c=(n&X)!==0,a=t,v=!0,w=()=>(v&&(v=!1,a=c?A(t):t),a),u;if(i){var d=x in e||T in e;u=E(e,r)?.set??(d&&r in e?o=>e[r]=o:void 0)}var f,m=!1;i?[f,m]=J(()=>e[r]):f=e[r],f===void 0&&t!==void 0&&(f=w(),u&&(s&&K(),u(f)));var l;if(s?l=()=>{var o=e[r];return o===void 0?w():(v=!0,o)}:l=()=>{var o=e[r];return o!==void 0&&(a=void 0),o===void 0?a:o},s&&(n&D)===0)return l;if(u){var _=e.$$legacy;return(function(o,S){return arguments.length>0?((!s||!S||_||m)&&u(S?l():o),o):l()})}var p=!1,h=((n&G)!==0?C:M)(()=>(p=!1,l()));i&&b(h);var $=R;return(function(o,S){if(arguments.length>0){const O=S?b(h):s&&i?U(o):o;return Y(h,O),p=!0,a!==void 0&&(a=O),o}return F&&p||($.f&q)!==0?h.v:b(h)})}const re="modulepreload",ne=function(e,r){return new URL(e,r).href},g={},oe=function(r,n,t){let s=Promise.resolve();if(n&&n.length>0){let w=function(u){return Promise.all(u.map(d=>Promise.resolve(d).then(f=>({status:"fulfilled",value:f}),f=>({status:"rejected",reason:f}))))};const c=document.getElementsByTagName("link"),a=document.querySelector("meta[property=csp-nonce]"),v=a?.nonce||a?.getAttribute("nonce");s=w(n.map(u=>{if(u=ne(u,t),u in g)return;g[u]=!0;const d=u.endsWith(".css"),f=d?'[rel="stylesheet"]':"";if(!!t)for(let _=c.length-1;_>=0;_--){const p=c[_];if(p.href===u&&(!d||p.rel==="stylesheet"))return}else if(document.querySelector(`link[href="${u}"]${f}`))return;const l=document.createElement("link");if(l.rel=d?"stylesheet":re,d||(l.as="script"),l.crossOrigin="",l.href=u,v&&l.setAttribute("nonce",v),document.head.appendChild(l),d)return new Promise((_,p)=>{l.addEventListener("load",_),l.addEventListener("error",()=>p(new Error(`Unable to preload CSS for ${u}`)))})}))}function i(c){const a=new Event("vite:preloadError",{cancelable:!0});if(a.payload=c,window.dispatchEvent(a),!a.defaultPrevented)throw c}return s.then(c=>{for(const a of c||[])a.status==="rejected"&&i(a.reason);return r().catch(i)})};export{oe as _,ie as b,ae as l,ee as p,ue as r,le as s};
