const __vite__mapDeps=(i,m=__vite__mapDeps,d=(m.f||(m.f=["../nodes/0.BmbMMmNK.js","../chunks/DsnmJJEf.js","../chunks/BLXpcXkd.js","../chunks/8VNsxgWr.js","../chunks/BreKPyJ7.js","../chunks/CACSdi0N.js","../chunks/DvOALBNi.js","../chunks/xrKURGDJ.js","../chunks/CdO6MHp9.js","../chunks/Cr2l1Gk0.js","../chunks/DLeuY3wH.js","../assets/0.ORHmqb1i.css","../nodes/1.rQknbWW3.js","../chunks/BmybysiL.js","../nodes/2.CgwGaiv7.js","../nodes/3.Ao-3nQmF.js","../chunks/Dvs79aKq.js","../chunks/D3MduT8N.js","../assets/View.DVDR0jJh.css","../assets/3.Dv6DhPRt.css","../nodes/4.BKrSHTw8.js"])))=>i.map(i=>d[i]);
import{p as E,b as P,_ as g}from"../chunks/CACSdi0N.js";import{x as b,aa as z,z as u,ag as B,ah as F,ai as K,B as N,aj as Y,v as H,l as J,N as Q,O as U,o as W,ak as x,al as X,q as I,D as y,I as Z,t as f,n as $,C as O,am as R,F as tt,G as et,an as rt,K as at,L as st}from"../chunks/BLXpcXkd.js";import"../chunks/DsnmJJEf.js";import{i as w}from"../chunks/8VNsxgWr.js";import{c as L}from"../chunks/CdO6MHp9.js";function ot(o){return class extends nt{constructor(t){super({component:o,...t})}}}class nt{#e;#t;constructor(t){var a=new Map,n=(e,r)=>{var c=H(r,!1,!1);return a.set(e,c),c};const i=new Proxy({...t.props||{},$$events:{}},{get(e,r){return u(a.get(r)??n(r,Reflect.get(e,r)))},has(e,r){return r===z?!0:(u(a.get(r)??n(r,Reflect.get(e,r))),Reflect.has(e,r))},set(e,r,c){return b(a.get(r)??n(r,c),c),Reflect.set(e,r,c)}});this.#t=(t.hydrate?B:F)(t.component,{target:t.target,anchor:t.anchor,props:i,context:t.context,intro:t.intro??!1,recover:t.recover}),(!t?.props?.$$host||t.sync===!1)&&K(),this.#e=i.$$events;for(const e of Object.keys(this.#t))e==="$set"||e==="$destroy"||e==="$on"||N(this,e,{get(){return this.#t[e]},set(r){this.#t[e]=r},enumerable:!0});this.#t.$set=e=>{Object.assign(i,e)},this.#t.$destroy=()=>{Y(this.#t)}}$set(t){this.#t.$set(t)}$on(t,a){this.#e[t]=this.#e[t]||[];const n=(...i)=>a.call(this,...i);return this.#e[t].push(n),()=>{this.#e[t]=this.#e[t].filter(i=>i!==n)}}$destroy(){this.#t.$destroy()}}const yt={};var it=I('<div id="svelte-announcer" aria-live="assertive" aria-atomic="true" style="position: absolute; left: 0; top: 0; clip: rect(0 0 0 0); clip-path: inset(50%); overflow: hidden; white-space: nowrap; width: 1px; height: 1px"><!></div>'),ct=I("<!> <!>",1);function ut(o,t){J(t,!0);let a=E(t,"components",23,()=>[]),n=E(t,"data_0",3,null),i=E(t,"data_1",3,null);Q(()=>t.stores.page.set(t.page)),U(()=>{t.stores,t.page,t.constructors,a(),t.form,n(),i(),t.stores.page.notify()});let e=x(!1),r=x(!1),c=x(null);W(()=>{const s=t.stores.page.subscribe(()=>{u(e)&&(b(r,!0),X().then(()=>{b(c,document.title||"untitled page",!0)}))});return b(e,!0),s});const j=R(()=>t.constructors[1]);var k=ct(),A=y(k);{var C=s=>{const m=R(()=>t.constructors[0]);var l=O(),h=y(l);L(h,()=>u(m),(d,_)=>{P(_(d,{get data(){return n()},get form(){return t.form},get params(){return t.page.params},children:(v,dt)=>{var D=O(),S=y(D);L(S,()=>u(j),(G,M)=>{P(M(G,{get data(){return i()},get form(){return t.form},get params(){return t.page.params}}),q=>a()[1]=q,()=>a()?.[1])}),f(v,D)},$$slots:{default:!0}}),v=>a()[0]=v,()=>a()?.[0])}),f(s,l)},T=s=>{const m=R(()=>t.constructors[0]);var l=O(),h=y(l);L(h,()=>u(m),(d,_)=>{P(_(d,{get data(){return n()},get form(){return t.form},get params(){return t.page.params}}),v=>a()[0]=v,()=>a()?.[0])}),f(s,l)};w(A,s=>{t.constructors[1]?s(C):s(T,!1)})}var V=Z(A,2);{var p=s=>{var m=it(),l=tt(m);{var h=d=>{var _=rt();at(()=>st(_,u(c))),f(d,_)};w(l,d=>{u(r)&&d(h)})}et(m),f(s,m)};w(V,s=>{u(e)&&s(p)})}f(o,k),$()}const bt=ot(ut),Et=[()=>g(()=>import("../nodes/0.BmbMMmNK.js"),__vite__mapDeps([0,1,2,3,4,5,6,7,8,9,10,11]),import.meta.url),()=>g(()=>import("../nodes/1.rQknbWW3.js"),__vite__mapDeps([12,1,9,2,3,4,13,10]),import.meta.url),()=>g(()=>import("../nodes/2.CgwGaiv7.js"),__vite__mapDeps([14,1,9,2,13,10]),import.meta.url),()=>g(()=>import("../nodes/3.Ao-3nQmF.js"),__vite__mapDeps([15,16,1,2,3,8,7,5,4,6,17,9,18,19]),import.meta.url),()=>g(()=>import("../nodes/4.BKrSHTw8.js"),__vite__mapDeps([20,1,9,2,17,7,5,4,18]),import.meta.url)],Pt=[],xt={"/":[2],"/edit":[3],"/view":[4]},mt={handleError:(({error:o})=>{console.error(o)}),reroute:(()=>{}),transport:{}},lt=Object.fromEntries(Object.entries(mt.transport).map(([o,t])=>[o,t.decode])),Ot=!1,Rt=(o,t)=>lt[o](t);export{Rt as decode,lt as decoders,xt as dictionary,Ot as hash,mt as hooks,yt as matchers,Et as nodes,bt as root,Pt as server_loads};
