import{b as u,h as s,a as _,E as m,c as h,d as v,e as d,s as b,f as E,p as g}from"./BLXpcXkd.js";function x(i,l,p){s&&_();var f=i,n,e,a=null,r=null;function t(){e&&(g(e),e=null),a&&(a.lastChild.remove(),f.before(a),a=null),e=r,r=null}u(()=>{if(n!==(n=l())){var c=b();if(n){var o=f;c&&(a=document.createDocumentFragment(),a.append(o=h()),e&&d.skipped_effects.add(e)),r=v(()=>p(o,n))}c?d.add_callback(t):t()}},m),s&&(f=E)}export{x as c};
