import"../chunks/DsnmJJEf.js";import{i as m}from"../chunks/Cr2l1Gk0.js";import{l as n,o as i,m as s,n as e,q as p,t as f}from"../chunks/BLXpcXkd.js";import{V as d}from"../chunks/D3MduT8N.js";import{i as l}from"../chunks/xrKURGDJ.js";var h=p('<meta name="robots" content="noindex"/>');function x(o,a){n(a,!1),i(l),m(),s(t=>{var r=h();f(t,r)}),d(o,{shouldShowGrid:!1}),e()}export{x as component};
